import { world, system, EntityDamageCause } from "@minecraft/server";

// MYCELIUM Armour

world.afterEvents.entityDie.subscribe((event) => {
    const deadEntity = event.deadEntity;
    const damageSource = event.damageSource.damagingEntity;
    if (!damageSource) { return; }
    if (damageSource.typeId === "minecraft:player" && damageSource.hasTag('dragons:mycelium_armour') && (deadEntity.matches({families: ['monster'] }) || deadEntity.matches({families: ['mob'] }) )) {
            damageSource.addEffect("absorption", 1200, {amplifier: 0, showParticles: true});
    }
});

// BAMBOO Armour

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    if (player.isSprinting && player.hasTag("dragons:bamboo_armour")) {
      player.addEffect("speed", 5, {amplifier: 1, showParticles: false});
      player.addEffect("jump_boost", 5, {amplifier: 0, showParticles: false});
    }
  }
});

// NIGHT Armour

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    if (player.isSneaking && player.hasTag("dragons:night_armour")) {
      player.addEffect("invisibility", 3, {amplifier: 0, showParticles: false});
      player.addEffect("speed", 3, {amplifier: 2, showParticles: false});
      player.playAnimation('animation.night_armour');
    }
  }
});

// DEADLY Armour

world.afterEvents.entityHurt.subscribe((event) => {
  const hurtEntity = event.hurtEntity;
  const damageSource = event.damageSource.damagingEntity;
  const cause = event.damageSource.cause
  const damage = event.damage
  if (!damageSource) {
    return;
  }
  if(!damageSource.isValid()) return;
  if (!hurtEntity.hasTag('dragons:deadly_armour')) {
    return;
  }
  if (hurtEntity == damageSource) {
    return;
  }
  if (cause == "thorns") return;

  damageSource.runCommand('particle dragons:deadly_deflect ~~1~');
  hurtEntity.playSound('damage.thorns', {
    pitch: 0.5,
    volume: 0.5
  });
  damageSource.applyDamage(damage*2, {
    cause: EntityDamageCause.thorns,
    damagingEntity: hurtEntity
  });

});

// ECHO Armour

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    if (player.hasTag("dragons:echo_armour")) {
      player.addEffect("night_vision", 200, {amplifier: 0, showParticles: false});
    }
  }
});

// FIRE Armour

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    if (player.hasTag("dragons:fire_armour")) {
      player.addEffect("fire_resistance", 10, {amplifier: 0, showParticles: false});
    }
  }
});

// NETHER Armour

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    if (player.hasTag("dragons:nether_armour")) {
      player.addEffect("fire_resistance", 10, {amplifier: 0, showParticles: false});
    }
  }
});

// QUAKE Armour

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    if (player.hasTag("dragons:quake_armour")) {
      player.addEffect("haste", 10, {amplifier: 1, showParticles: false});
    }
  }
});

// SNOW Armour

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    if (player.hasTag("dragons:snow_armour")) {
      player.addEffect("resistance", 10, {amplifier: 0, showParticles: false});
    }
  }
});

// SKY Armour

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    if (player.hasTag("dragons:sky_armour")) {
      player.addEffect("speed", 10, {amplifier: 0, showParticles: false});
    }
  }
});

// VOID Armour

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    if (player.hasTag("dragons:void_armour")) {
      player.addEffect("strength", 10, {amplifier: 0, showParticles: false});
    }
  }
});

// TIDE Armour

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    if (player.hasTag("dragons:tide_armour")) {
      player.addEffect("water_breathing", 10, {amplifier: 0, showParticles: false});
    }
  }
});

// ENCHANTED Armour

const positiveEffects = [
  'speed',
  'haste',
  'strength',
  'jump_boost',
  'regeneration',
  'resistance',
  'health_boost',
  'absorption'
];

world.afterEvents.effectAdd.subscribe((event) => {
  const entity = event.entity;
  if (entity.hasTag('ignoreEnchanted')) {
    entity.removeTag('ignoreEnchanted')
    return;
  }
  const amplifier = event.effect.amplifier;
  const duration = event.effect.duration;
  const effectId = event.effect.typeId;
  if (amplifier >=255 || entity.typeId !== 'minecraft:player') return;
  if (!positiveEffects.includes(effectId)) {
    return;
  }
  if(!entity.hasTag('dragons:enchanted_armour')) return;
  entity.addTag('ignoreEnchanted')
  entity.removeEffect(effectId);
  entity.addEffect(effectId, duration, {
    amplifier: amplifier+1
  });
  system.runTimeout(() => {
    entity.removeTag('ignoreEnchanted')
  }, 3);
});

// FOREST Armour

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    if (player.hasTag("dragons:forest_armour")) {
      let hp = player.getComponent('minecraft:health')
      hp.setCurrentValue( hp.currentValue + 1)
    }
  }
}, 100);